'use strict';

exports.__esModule = true;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var React = _interopRequireWildcard(_react);

var _BrowserDetection = require('../utils/BrowserDetection');

var _utils = require('../utils');

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _defaults(obj, defaults) { var keys = Object.getOwnPropertyNames(defaults); for (var i = 0; i < keys.length; i++) { var key = keys[i]; var value = Object.getOwnPropertyDescriptor(defaults, key); if (value && value.configurable && obj[key] === undefined) { Object.defineProperty(obj, key, value); } } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : _defaults(subClass, superClass); }

var sizerStyle = {
  position: 'absolute',
  top: 0,
  left: 0,
  visibility: 'hidden',
  height: 0,
  overflow: 'scroll',
  whiteSpace: 'pre'
};

var copyStyles = function copyStyles(styles, node) {
  node.style.fontSize = styles.fontSize;
  node.style.fontFamily = styles.fontFamily;
  node.style.fontWeight = styles.fontWeight;
  node.style.fontStyle = styles.fontStyle;
  node.style.letterSpacing = styles.letterSpacing;
  node.style.textTransform = styles.textTransform;
};

var InputAutosize = function (_React$Component) {
  _inherits(InputAutosize, _React$Component);

  function InputAutosize(props) {
    _classCallCheck(this, InputAutosize);

    var _this = _possibleConstructorReturn(this, _React$Component.call(this, props));

    _this.mounted = null;
    _this.input = null;
    _this.placeHolderSizer = null;
    _this.sizer = null;

    _this.bindInputRef = function (ref) {
      _this.input = ref;
      var inputRef = _this.props.inputRef;

      inputRef && inputRef(ref);
    };

    _this.bindPlaceHolderSizerRef = function (ref) {
      _this.placeHolderSizer = ref;
    };

    _this.bindSizerRef = function (ref) {
      _this.sizer = ref;
    };

    _this.state = {
      inputWidth: props.minWidth
    };
    return _this;
  }

  InputAutosize.prototype.componentDidMount = function componentDidMount() {
    this.mounted = true;
    this.copyInputStyles();
    this.updateInputWidth();
  };

  InputAutosize.prototype.componentDidUpdate = function componentDidUpdate(prevProps, prevState) {
    var inputWidth = this.state.inputWidth;
    var onAutosize = this.props.onAutosize;

    if (prevState.inputWidth !== inputWidth) {
      onAutosize && onAutosize(inputWidth);
    }
    this.updateInputWidth();
  };

  InputAutosize.prototype.componentWillUnmount = function componentWillUnmount() {
    this.mounted = false;
  };

  InputAutosize.prototype.focus = function focus() {
    if (this.input) {
      this.input.focus();
    }
  };

  InputAutosize.prototype.blur = function blur() {
    if (this.input) {
      this.input.blur();
    }
  };

  InputAutosize.prototype.copyInputStyles = function copyInputStyles() {
    if (!this.mounted || !window.getComputedStyle) {
      return;
    }
    var inputStyles = this.input && window.getComputedStyle(this.input);
    if (!inputStyles) {
      return;
    }
    if (this.sizer) {
      copyStyles(inputStyles, this.sizer);
    }

    if (this.placeHolderSizer) {
      copyStyles(inputStyles, this.placeHolderSizer);
    }
  };

  InputAutosize.prototype.updateInputWidth = function updateInputWidth() {
    if (!this.mounted || !this.sizer || typeof this.sizer.scrollWidth === 'undefined') {
      return;
    }
    var _props = this.props,
        minWidth = _props.minWidth,
        placeholder = _props.placeholder,
        value = _props.value;

    var newInputWidth = void 0;
    if (placeholder && !value && this.placeHolderSizer) {
      newInputWidth = Math.max(this.sizer.scrollWidth, this.placeHolderSizer.scrollWidth) + 2;
    } else {
      newInputWidth = this.sizer.scrollWidth + 2;
    }

    if (newInputWidth < minWidth) {
      newInputWidth = minWidth;
    }
    if (newInputWidth !== this.state.inputWidth) {
      this.setState({
        inputWidth: newInputWidth
      });
    }
  };

  InputAutosize.prototype.renderStyles = function renderStyles() {
    var inputId = this.props.inputId;


    return _BrowserDetection.isIE ? React.createElement('style', {
      dangerouslySetInnerHTML: {
        __html: 'input#' + inputId + '::-ms-clear {display: none;}'
      }
    }) : null;
  };

  InputAutosize.prototype.render = function render() {
    var inputWidth = this.state.inputWidth;
    var _props2 = this.props,
        defaultValue = _props2.defaultValue,
        value = _props2.value,
        style = _props2.style,
        className = _props2.className,
        placeholder = _props2.placeholder,
        inputClassName = _props2.inputClassName,
        inputStyle = _props2.inputStyle,
        inputId = _props2.inputId;


    var sizerValue = [defaultValue, value, ''].reduce(function (previousValue, currentValue) {
      if (previousValue !== null && previousValue !== undefined) {
        return previousValue;
      }
      return currentValue;
    });

    var wrapperStyle = _extends({}, style);
    if (!wrapperStyle.display) {
      wrapperStyle.display = 'inline-block';
    }

    var nextInputStyle = _extends({
      boxSizing: 'content-box',
      width: inputWidth + 'px'
    }, inputStyle);

    var _partitionHTMLProps = (0, _utils.partitionHTMLProps)(this.props),
        htmlInputProps = _partitionHTMLProps[0];

    htmlInputProps.className = inputClassName;
    htmlInputProps.id = inputId;
    htmlInputProps.style = nextInputStyle;

    return React.createElement(
      'div',
      { className: className, style: wrapperStyle },
      this.renderStyles(),
      React.createElement('input', _extends({}, htmlInputProps, { ref: this.bindInputRef, type: 'text' })),
      React.createElement(
        'div',
        { ref: this.bindSizerRef, style: sizerStyle },
        sizerValue
      ),
      placeholder ? React.createElement(
        'div',
        { ref: this.bindPlaceHolderSizerRef, style: sizerStyle },
        placeholder
      ) : null
    );
  };

  return InputAutosize;
}(React.Component);

InputAutosize.defaultProps = {
  minWidth: 1,
  inputId: '_' + Math.random().toString(36).substr(2, 12)
};
InputAutosize.handledProps = ['className', 'defaultValue', 'inputClassName', 'inputId', 'inputRef', 'inputStyle', 'minWidth', 'onAutosize', 'onChange', 'placeholder', 'style', 'value'];
InputAutosize.propTypes = {
  className: _propTypes2.default.string,
  defaultValue: _propTypes2.default.any,
  inputId: _propTypes2.default.string.isRequired,
  inputClassName: _propTypes2.default.string,
  inputRef: _propTypes2.default.func,
  inputStyle: _propTypes2.default.object,
  minWidth: _propTypes2.default.number.isRequired,
  onAutosize: _propTypes2.default.func,
  onChange: _propTypes2.default.func,
  placeholder: _propTypes2.default.string,
  style: _propTypes2.default.object,
  value: _propTypes2.default.any
};
exports.default = InputAutosize;