'use strict';

exports.__esModule = true;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var React = _interopRequireWildcard(_react);

var _classnames = require('classnames');

var _classnames2 = _interopRequireDefault(_classnames);

var _Transition = require('rsuite-utils/lib/Animation/Transition');

var _Transition2 = _interopRequireDefault(_Transition);

var _domLib = require('dom-lib');

var _utils = require('./utils');

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _defaults(obj, defaults) { var keys = Object.getOwnPropertyNames(defaults); for (var i = 0; i < keys.length; i++) { var key = keys[i]; var value = Object.getOwnPropertyDescriptor(defaults, key); if (value && value.configurable && obj[key] === undefined) { Object.defineProperty(obj, key, value); } } return obj; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : _defaults(subClass, superClass); }

var Ripple = function (_React$Component) {
  _inherits(Ripple, _React$Component);

  function Ripple(props) {
    _classCallCheck(this, Ripple);

    var _this = _possibleConstructorReturn(this, _React$Component.call(this, props));

    _this.mousedownListener = null;
    _this.trigger = null;

    _this.bindTriggerRef = function (ref) {
      _this.trigger = ref;
    };

    _this.getPosition = function (event) {
      var offset = (0, _domLib.getOffset)(_this.trigger);
      var offsetX = (event.pageX || 0) - offset.left;
      var offsetY = (event.pageY || 0) - offset.top;

      var radiusX = Math.max(offset.width - offsetX, offsetX);
      var radiusY = Math.max(offset.height - offsetY, offsetY);
      var radius = Math.sqrt(Math.pow(radiusX, 2) + Math.pow(radiusY, 2));

      return {
        width: radius * 2,
        height: radius * 2,
        left: offsetX - radius,
        top: offsetY - radius
      };
    };

    _this.onMouseDown = function (event) {
      var position = _this.getPosition(event);
      var onMouseDown = _this.props.onMouseDown;


      _this.setState({
        rippling: true,
        position: position
      });

      onMouseDown && onMouseDown(position, event);
    };

    _this.handleRippled = function () {
      _this.setState({
        rippling: false
      });
    };

    _this.addPrefix = function (name) {
      return (0, _utils.prefix)(_this.props.classPrefix)(name);
    };

    _this.state = {
      rippling: false,
      position: {}
    };
    return _this;
  }

  Ripple.prototype.componentDidMount = function componentDidMount() {
    if (this.trigger) {
      this.mousedownListener = (0, _domLib.on)(this.trigger.parentNode, 'mousedown', this.onMouseDown);
    }
  };

  Ripple.prototype.componentWillUnmount = function componentWillUnmount() {
    if (this.mousedownListener) {
      this.mousedownListener.off();
    }
  };

  Ripple.prototype.render = function render() {
    var _props = this.props,
        className = _props.className,
        classPrefix = _props.classPrefix,
        props = _objectWithoutProperties(_props, ['className', 'classPrefix']);

    var classes = (0, _classnames2.default)(this.addPrefix('pond'), className);
    var _state = this.state,
        position = _state.position,
        rippling = _state.rippling;


    return React.createElement(
      'span',
      _extends({}, props, { className: classes, ref: this.bindTriggerRef }),
      React.createElement(
        _Transition2.default,
        {
          'in': rippling,
          enteringClassName: this.addPrefix('rippling'),
          onEntered: this.handleRippled
        },
        React.createElement('span', { className: classPrefix, style: position })
      )
    );
  };

  return Ripple;
}(React.Component);

Ripple.handledProps = ['className', 'classPrefix', 'onMouseDown'];
Ripple.propTypes = {
  classPrefix: _propTypes2.default.string,
  className: _propTypes2.default.string,
  onMouseDown: _propTypes2.default.func
};
exports.default = (0, _utils.defaultProps)({
  classPrefix: 'ripple'
})(Ripple);